@@ -1,22 +1,22 @@
 /* builtins.h -- What a builtin looks like, and where to find them. */
 
-/* Copyright (C) 1987,1991 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2009 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
-   Bash is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   Bash is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   Bash is distributed in the hope that it will be useful, but WITHOUT
-   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-   License for more details.
+   Bash is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Bash; see the file COPYING.  If not, write to the Free
-   Software Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
+   along with Bash.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "config.h"
 
@@ -35,11 +35,12 @@
 #endif
 
 /* Flags describing various things about a builtin. */
-#define BUILTIN_ENABLED 0x1	/* This builtin is enabled. */
-#define BUILTIN_DELETED 0x2	/* This has been deleted with enable -d. */
-#define STATIC_BUILTIN  0x4	/* This builtin is not dynamically loaded. */
-#define SPECIAL_BUILTIN 0x8	/* This is a Posix `special' builtin. */
+#define BUILTIN_ENABLED 0x01	/* This builtin is enabled. */
+#define BUILTIN_DELETED 0x02	/* This has been deleted with enable -d. */
+#define STATIC_BUILTIN  0x04	/* This builtin is not dynamically loaded. */
+#define SPECIAL_BUILTIN 0x08	/* This is a Posix `special' builtin. */
 #define ASSIGNMENT_BUILTIN 0x10	/* This builtin takes assignment statements. */
+#define POSIX_BUILTIN	0x20	/* This builtins is special in the Posix command search order. */
 
 #define BASE_INDENT	4
 
