@@ -42,10 +42,11 @@
    Used by alias and trap, among others. */
 char *
 sh_single_quote (string)
-     char *string;
+     const char *string;
 {
   register int c;
-  char *result, *r, *s;
+  char *result, *r;
+  const char *s;
 
   result = (char *)xmalloc (3 + (4 * strlen (string)));
   r = result;
@@ -72,10 +73,11 @@ sh_single_quote (string)
 /* Quote STRING using double quotes.  Return a new string. */
 char *
 sh_double_quote (string)
-     char *string;
+     const char *string;
 {
   register unsigned char c;
-  char *result, *r, *s;
+  char *result, *r;
+  const char *s;
 
   result = (char *)xmalloc (3 + (2 * strlen (string)));
   r = result;
