@@ -4,19 +4,19 @@
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
-   Bash is free software; you can redistribute it and/or modify it under
-   the terms of the GNU General Public License as published by the Free
-   Software Foundation; either version 2, or (at your option) any later
-   version.
+   Bash is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   Bash is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or
-   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-   for more details.
+   Bash is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with Bash; see the file COPYING.  If not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA */
+   You should have received a copy of the GNU General Public License
+   along with Bash.  If not, see <http://www.gnu.org/licenses/>.
+*/
    
 #include <config.h>
 
@@ -24,11 +24,7 @@
 
 #include <stdc.h>
 #include <bashansi.h>
-#include <ctype.h>
-
-#if !defined (to_lower)
-#  define to_lower(c) (islower(c) ? (c) : tolower(c))
-#endif /* to_lower */
+#include <chartypes.h>
 
 /* Compare at most COUNT characters from string1 to string2.  Case
    doesn't matter. */
@@ -49,7 +45,7 @@ strncasecmp (string1, string2, count)
   s2 = string2;
   do
     {
-      if ((r = to_lower (*s1) - to_lower (*s2)) != 0)
+      if ((r = TOLOWER ((unsigned char) *s1) - TOLOWER ((unsigned char) *s2)) != 0)
 	return r;
       if (*s1++ == '\0')
 	break;
@@ -76,7 +72,7 @@ strcasecmp (string1, string2)
   if (s1 == s2)
     return (0);
 
-  while ((r = to_lower (*s1) - to_lower (*s2)) == 0)
+  while ((r = TOLOWER ((unsigned char)*s1) - TOLOWER ((unsigned char)*s2)) == 0)
     {
       if (*s1++ == '\0')
 	return 0;
