@@ -1,22 +1,22 @@
 /* stringlib.c - Miscellaneous string functions. */
 
-/* Copyright (C) 1996-2002 Free Software Foundation, Inc.
+/* Copyright (C) 1996-2009 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
-   Bash is free software; you can redistribute it and/or modify it under
-   the terms of the GNU General Public License as published by the Free
-   Software Foundation; either version 2, or (at your option) any later
-   version.
+   Bash is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   Bash is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or
-   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-   for more details.
+   Bash is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License along
-   with Bash; see the file COPYING.  If not, write to the Free Software
-   Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */
+   You should have received a copy of the GNU General Public License
+   along with Bash.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "config.h"
 
@@ -125,7 +125,7 @@ find_index_in_alist (string, alist, flags)
    not including END. */
 char *
 substring (string, start, end)
-     char *string;
+     const char *string;
      int start, end;
 {
   register int len;
@@ -170,7 +170,10 @@ strsub (string, pat, rep, global)
 	  temp[templen++] = string[i++];
 	}
     }
-  temp[templen] = 0;
+  if (temp)
+    temp[templen] = 0;
+  else
+    temp = savestring (string);
   return (temp);
 }
 
